# -*- coding: utf-8 -*-
# SPDX-FileCopyrightText: : 2023 The PyPSA-Eur Authors
#
# SPDX-License-Identifier: MIT
"""
Retrieve monthly fuel prices from Destatis.
"""

import logging

logger = logging.getLogger(__name__)

from pathlib import Path

from _helpers import configure_logging, progress_retrieve

if __name__ == "__main__":
    if "snakemake" not in globals():
        from _helpers import mock_snakemake

        snakemake = mock_snakemake("retrieve_monthly_fuel_prices")
        rootpath = ".."
    else:
        rootpath = "."
    configure_logging(snakemake)

    url = "https://www.destatis.de/EN/Themes/Economy/Prices/Publications/Downloads-Energy-Price-Trends/energy-price-trends-xlsx-5619002.xlsx?__blob=publicationFile"

    to_fn = Path(rootpath) / Path(snakemake.output[0])

    logger.info(f"Downloading monthly fuel prices from '{url}'.")
    disable_progress = snakemake.config["run"].get("disable_progressbar", False)
    progress_retrieve(url, to_fn, disable=disable_progress)

    logger.info(f"Monthly fuel prices available at {to_fn}")
